@@ -18,6 +18,7 @@ class MyFigures
         @figure_maker = FigureMaker.default
                 
         t.save_dir = 'figures_out'
+        # t.autocleanup = false
         t.def_eval_function { |str| eval(str) }
 
         t.def_figure("Icon") { icon }
@@ -57,7 +58,14 @@ class MyFigures
         t.def_figure("Colors2") { colors2 }
         t.def_figure("Colors3") { colors3 }
         t.def_figure("Colors4") { colors4 }
+
+
+        t.def_figure("Tex_Fonts") { tex_fonts }
+
+        t.def_figure("Text_size") { text_size }
+        t.def_figure("Text_size_with_rotation") { text_size_with_rotation }
         
+
         hues
         
         t.def_enter_page_function { enter_page }
@@ -206,6 +214,23 @@ class MyFigures
             t.show_marker('font' => i+1, 'text' => 'abc', 'at' => [0.5+dx, 0.5+dy], 'scale' => 1.25)
         end
     end
+
+
+    def tex_fonts
+      background
+      t.show_text('text' => 'Various fonts (currently not working)', 
+                  'side' => TOP, 'shift' => 0.6,
+                  'justification' => CENTERED, 
+                  'scale' => 0.9, 'position' => 0.5)
+      
+      t.show_text('text' => 'Default font',
+                  'x' => 0.02, 'y' => 0.93,
+                  'justification' => LEFT_JUSTIFIED )
+      t.tex_fontfamily = "sfdefault"
+      t.show_text('text' => 'Default sans serif',
+                  'x' => 0.02, 'y' => 0.86,
+                  'justification' => LEFT_JUSTIFIED )
+    end
     
     def rounded_rect
         t.show_text('text' => 'append\_rounded\_rect\_to\_path', 'side' => TOP, 'shift' => 0.6,
@@ -220,7 +245,7 @@ class MyFigures
         t.stroke
         t.append_rounded_rect_to_path(x, y, width, height, dx, dy)
         t.clip
-        jpg_image
+        jpg_image_transparent
     end
     
     def arc
@@ -316,6 +341,143 @@ class MyFigures
                 'scale' => 0.7, 'angle' => t.convert_to_degrees(dx,dy)) 
         end
     end
+
+    # In this example, we show how we can use measures of the text
+    # transparently with Tioga:
+    def text_size
+      t.stroke_rect(0,0,1,1)
+      t.rescale(0.5)
+      
+      tries = [
+               {
+                 'text' => 'Using side/position instead of X and Y', 
+                 'side' => TOP, 'shift' => 0.6,
+                 'justification' => CENTERED, 
+                 'scale' => 2, 'position' => 0.5
+               },
+               {
+                 'x' => 0.2, 'y' => 0.8,
+                 'justification' => LEFT_JUSTIFIED,
+                 'alignment' => ALIGNED_AT_TOP,
+                 'scale' => 4,
+                 'text' => 'top left'
+               },
+               {
+                 'x' => 0.3, 'y' => 0.5,
+                 'justification' => RIGHT_JUSTIFIED,
+                 'alignment' => ALIGNED_AT_BOTTOM,
+                 'scale' => 2,
+                 'text' => 'bottom right'
+               },
+               {
+                 'x' => 0.3, 'y' => 0.2,
+                 'justification' => CENTERED,
+                 'alignment' => ALIGNED_AT_BASELINE,
+                 'scale' => 2.3,
+                 'text' => 'centered \raise-.5ex\hbox{b}aseline'
+               },
+               {
+                 'x' => 0.7, 'y' => 0.6,
+                 'justification' => LEFT_JUSTIFIED,
+                 'alignment' => ALIGNED_AT_MIDHEIGHT,
+                 'scale' => 2,
+                 'text' => 'left mid-height'
+               },
+               {
+                 'x' => 0.7, 'y' => 0.9,
+                 'justification' => CENTERED,
+                 'alignment' => ALIGNED_AT_MIDHEIGHT,
+                 'scale' => 2,
+                 'text' => 'rotated',
+                 'angle' => -30
+               },
+               {
+                 'x' => 0.5, 'y' => 0.4,
+                 'justification' => RIGHT_JUSTIFIED,
+                 'alignment' => ALIGNED_AT_BASELINE,
+                 'scale' => 2,
+                 'text' => 'a right-justified rotated text',
+                 'angle' => 12
+               },
+               {
+                 'x' => 1, 'y' => 0.4,
+                 'justification' => LEFT_JUSTIFIED,
+                 'alignment' => ALIGNED_AT_TOP,
+                 'scale' => 2,
+                 'text' => 'a left-justified rotated text',
+                 'angle' => 240
+               },
+               
+              ]
+      i = 0
+      for try in tries
+        a = { 'scale' => 1, 
+          'measure' => "box#{i}"
+        }
+        a.update(try)
+        t.show_text(a)
+        size = t.get_text_size("box#{i}")
+        i += 1
+
+        # We draw a box around the text:
+        if size.key? 'width'
+          xs = Dvector.new
+          ys = Dvector.new
+          for x,y in size['points']
+            xs << t.convert_output_to_figure_x(10*x)
+            ys << t.convert_output_to_figure_y(10*y)
+          end
+          xs << xs[0]
+          ys << ys[0]
+          t.show_polyline(xs,ys, Red, nil, Line_Type_Dot)
+        end
+        t.show_marker({
+                        'x' => try['x'],
+                        'y' => try['y'],
+                        'marker' => MarkerConstants::Plus,
+                        'scale' => 0.6,
+                        'color' => Green,
+                        'angle' => try['angle'] || 0
+                      })
+      end
+    end
+
+    # In this example, we show how we can use measures of the text
+    # transparently with Tioga:
+    def text_size_with_rotation
+      t.stroke_rect(0,0,1,1)
+      t.rescale(0.5)
+
+      equation = '\int_{-\infty}^{\infty} \! e^{-x^{2}}\, \! dx = \sqrt{\pi}'
+      text = "\\fbox{$\\displaystyle #{equation}$}"
+
+      nb = 5
+      nb.times do |i|
+        scale = 0.5 + i * 0.2
+        angle = i * 37
+        x = (i+1)/(nb+1.0)
+        y = x
+        color = [1.0 - i * 0.2, i*0.2, 0]
+        t.show_text('text' => text, 'color' => color, 'x' => x, 
+                    'y' => x,
+                    'alignment' => ALIGNED_AT_MIDHEIGHT,
+                    'scale' => scale , 'measure' => "box#{i}", 
+                    'angle' => angle )
+        size = t.get_text_size("box#{i}")
+        if size.key? 'points'
+          xs = Dvector.new
+          ys = Dvector.new
+          for x,y in size['points']
+            xs << t.convert_output_to_figure_x(10*x)
+            ys << t.convert_output_to_figure_y(10*y)
+          end
+          t.stroke_color = color
+          t.line_type = Line_Type_Dashes
+          t.stroke_rect(xs.min, ys.min,
+                        (xs.max - xs.min),(ys.max - ys.min))
+        end
+      end
+    end
     
     def gradient_colormap
         t.landscape
@@ -780,7 +942,7 @@ class MyFigures
     def jpg_image_transparent
         t.show_image(
             'jpg' => "data/cassini.jpg", 'width' => 999, 'height' => 959,
-            'll' => [0.01, 0.01], 'lr' => [0.99, 0.01], 'ul' => [0.01, 0.99]) 
+            'll' => [0.01, 0.01], 'lr' => [0.99, 0.01], 'ul' => [0.01, 0.99])
         t.fill_opacity = 0.6
         t.show_image( # Lucy amazed by Io
             'jpg' => "data/lucy.jpg", 'width' => 148, 'height' => 164,
@@ -841,7 +1003,7 @@ class MyFigures
         t.show_image(
             'jpg' => "data/lucy.jpg", 'width' => 148, 'height' => 164,
             'll' => [0.99, 0.01], 'lr' => [0.01, 0.01], 'ul' => [0.99, 0.99])
-        samples = t.create_monochrome_image_data(build_table(0.1, 0.9), 'boundary' => 0.5)
+        samples = t.create_monochrome_image_data(build_table(0.1, 0.9), 'boundary' => 0.5)        
         dict = Hash.new
         dict['interpolate'] = false
         dict['ll'] = [0.1, 0.1]
@@ -1272,7 +1434,7 @@ class MyFigures
         t.set_portrait
         background
         t.rescale(0.45)
-        dy = -t.default_text_height_dy * 1.7
+        dy = -t.default_text_height_dy * 1.95
         t.fill_color = Black
         x = 0.24; y = y_init = 0.94
         list = [ 'Arrow', 'Arrowhead', 'Asterisk', 'Bar', 'BarThick', 'BarThin', 'Box', 'Bullet', 
