@@ -38,10 +38,13 @@ class MyPlots
         t.def_figure("Blues") { blues }
         t.def_figure("Reds") { reds }
         t.def_figure("Greens") { greens }
+        t.def_figure("Blues_axes_1") { blues_axes_1 }
+        t.def_figure("Blues_axes_2") { blues_axes_2 }
         t.def_figure("Log_Reds") { log_reds }
         t.def_figure("Side_by_Side") { side_by_side }
         t.def_figure("Two_Ys") { two_yaxes }
         t.def_figure("Legends") { legends }
+        t.def_figure("Two_Ys_with_Legends") { two_yaxes_with_legends }
         t.def_figure("Reds_and_Blues") { reds_blues }
         t.def_figure("Legend_Inside") { legend_inside }
         t.def_figure("Legend_Outside") { legend_outside }
@@ -58,6 +61,8 @@ class MyPlots
         t.def_figure("Blues_reversed_axes") { blues_reversed_axes }
         t.def_figure("Labels") { labels }
         t.def_figure("Error_Bars") { error_bars }
+        t.def_figure("Error_Bars_X") { error_bars_x }
+        t.def_figure("Error_Bars_Y") { error_bars_y }
         t.def_figure("Error_Bars2") { error_bars2 }
         t.def_figure("Arrows") { arrows }
         t.def_figure("Special_Y") { special_y }
@@ -67,6 +72,12 @@ class MyPlots
         t.def_figure("Sampled_Data") { sampled_data }
         t.def_figure("Contours") { samples_with_contours }
 
+        t.def_figure("Greens_clipped") { greens_clipped }
+
+
+        # Starting to play with axes specifications
+        t.def_figure("Axes_fun") { axes_fun }
+
         t.model_number = -1
         
         t.def_enter_page_function { enter_page }
@@ -142,6 +153,7 @@ EOD
     
     def blues
         read_data
+        t.title_color = Crimson
         t.do_box_labels('Blues Plot', 'Position', '\textcolor[rgb]{0,0,1}{Blues}')
         show_model_number
         xs = @positions
@@ -149,6 +161,36 @@ EOD
         t.show_plot(plot_boundaries(xs,ys,@margin,-1,1)) { 
             t.show_polyline(xs,ys,Blue) }
     end
+
+    def blues_axes_1
+      read_data
+      t.title_color = Crimson
+      t.do_box_labels('Blues Plot --- without top and right', 
+                      'Position', '\textcolor[rgb]{0,0,1}{Blues}')
+      show_model_number
+      xs = @positions
+      ys = @blues
+      t.top_edge_visible = false
+      t.right_edge_visible = false
+      t.show_plot(plot_boundaries(xs,ys,@margin,-1,1)) { 
+        t.show_polyline(xs,ys,Blue) }
+    end
+
+    def blues_axes_2
+      read_data
+      t.title_color = Crimson
+      t.do_box_labels('Blues Plot', 
+                      'Position', '\textcolor[rgb]{0,0,1}{Blues}')
+      show_model_number
+      xs = @positions
+      ys = @blues
+      t.top_edge_visible = false
+      t.right_edge_visible = false
+      t.bottom_edge_visible = false
+      t.xaxis_loc = AT_Y_ORIGIN
+      t.show_plot(plot_boundaries(xs,ys,@margin,-1,1)) { 
+        t.show_polyline(xs,ys,Blue) }
+    end
     
     def blues_reversed_axes # reverse both axes
         read_data
@@ -208,6 +250,23 @@ EOD
         greens_min = -4
         greens_max = -0.01
         boundaries = plot_boundaries(xs,ys,@margin,greens_min,greens_max)
+        t.show_plot(boundaries) { 
+            t.show_polyline(xs,ys,Green) 
+         }
+    end
+
+
+    # Same thing as green, but with X/Y clipped:
+    def greens_clipped
+        read_data
+        show_model_number
+        t.do_box_labels('Greens Plot Clipped', 'Position', 
+                        '\textcolor[rgb]{0,0.5,0}{Greens, clipped}')
+        xs = @positions
+        ys = @greens
+        greens_min = -4
+        greens_max = -0.01
+        boundaries = [ 1, 4, 0, -4 ]
         t.show_plot(boundaries) { t.show_polyline(xs,ys,Green) }
     end
     
@@ -248,6 +307,32 @@ EOD
             t.left_edge_type = AXIS_HIDDEN; greens }
         show_model_number
     end
+
+
+    def two_yaxes_with_legends
+      read_data
+      t.do_box_labels('Same X, Different Y\'s', 'Position', nil)
+      t.show_plot_with_legend('legend_left_margin' => 0.65,
+                              'plot_scale' => 1,
+                              'legend_scale' => 1.3,
+                              'plot_right_margin' => 0) { 
+        t.subplot {
+          t.yaxis_loc = t.ylabel_side = LEFT;
+          t.right_edge_type = AXIS_HIDDEN; 
+          t.show_plot(plot_boundaries(@positions,@blues,@margin,-1,1)) {
+            t.show_polyline(@positions,@blues,Blue,'Blues')
+          }
+        }
+        t.subplot {
+          t.yaxis_loc = t.ylabel_side = RIGHT;
+          t.left_edge_type = AXIS_HIDDEN; 
+          t.show_plot(plot_boundaries(@positions,@reds,@margin,-1,1)) {
+            t.show_polyline(@positions,@reds,Red,'Reds')
+          }
+        }
+      }
+      show_model_number
+    end
     
     def legend_inside
         read_data
@@ -262,7 +347,17 @@ EOD
     def legend_outside
         read_data
         show_model_number
-        t.show_plot_with_legend('legend_scale' => 1.3) { reds_blues }
+        t.show_plot_with_legend('legend_scale' => 1.3) {
+            reds_blues
+            t.show_marker(
+                'marker' => Circle,
+                'scale' => 0.5,
+                'fill_color' => Teal,
+                'Xs' => [0,2,4,6], 
+                'Ys' => [0.2,-0.1,0.3,-0.5],
+                'legend' => 'Spots'
+                )
+        }
     end
 
     def inset
@@ -442,13 +537,14 @@ EOD
         t.rescale(0.5)
         t.no_xlabel
         t.line_width = 0.5
+        t.title_shift = 0.5
         t.subplot(
             'left_margin' => 0.1, 'right_margin' => 0.5, 
-            'bottom_margin' => 0.75) { t.ylabel_angle = -90; side_by_side }
+            'bottom_margin' => 0.75) { side_by_side }
         t.yaxis_loc = RIGHT
         t.subplot(
             'left_margin' => 0.75, 'right_margin' => 0.07, 
-            'top_margin' => 0.2, 'bottom_margin' => 0.55) { reds }
+            'top_margin' => 0.2, 'bottom_margin' => 0.55) { t.ylabel_shift = 0.3; reds }
         t.yaxis_loc = LEFT
         t.subplot(
             'right_margin' => 0.7, 'top_margin' => 0.4) { rows }
@@ -532,6 +628,24 @@ EOD
         end
     end
     
+    def error_bars_y
+        xs = @positions
+        ys = @blues
+        do_labels_plot("Error Bars on Points") do
+            t.show_error_bars('x' => xs[@pt2], 'y' => ys[@pt2], 'dy' => 0.12)
+            t.show_error_bars('x' => xs[@pt1], 'y' => ys[@pt1], 'dy' => 0.15, 'color' => Crimson)
+        end
+    end
+    
+    def error_bars_x
+        xs = @positions
+        ys = @blues
+        do_labels_plot("Error Bars on Points") do
+            t.show_error_bars('x' => xs[@pt2], 'y' => ys[@pt2], 'dx' => 0.2)
+            t.show_error_bars('x' => xs[@pt1], 'y' => ys[@pt1], 'dx' => 0.23, 'color' => Crimson)
+        end
+    end
+    
     def error_bars2
         t.do_box_labels("Another Example of Error Bars", "Number of Particles", "Crossing Times")
         xs = Dvector[3, 4, 5, 6, 7, 8, 9, 12]
@@ -571,7 +685,7 @@ EOD
         x_first = 0.0; x_last = 18.0; y_first = y_last = 2.5
         x_results = Dvector[]
         y_results = Dvector[]
-        t.make_steps(
+        stps = t.make_steps(
             'dest_xs' => x_results, 'dest_ys' => y_results, 
             'xs' => xs, 'ys' => ys,
             'x_first' => x_first, 'y_first' => y_first, 
@@ -582,9 +696,8 @@ EOD
             smooth_pts = 4*(data_pts-1) + 1
             dx = (xs[data_pts-1] - xs[0])/(smooth_pts-1)
             sample_xs = Dvector.new(smooth_pts) { |i| i*dx + xs[0] }
-            result_ys = Dvector.new
-            t.make_spline_interpolated_points(
-                'sample_xs' => sample_xs, 'result_ys' => result_ys,
+            result_ys = t.make_spline_interpolated_points(
+                'sample_xs' => sample_xs,
                 'xs' => xs, 'ys' => ys,
                 'start_slope' => 2.5*(ys[1]-ys[0])/(xs[1]-xs[0]))
             t.stroke_color = Blue
@@ -603,10 +716,9 @@ EOD
         ys = Dvector[ 3.0, 3.7, 3.9, 4.2, 5.7, 6.6,  7.1,  6.7,  4.5 ]
         data_pts = xs.size
         x_first = 0.0; x_last = 18.0; y_first = y_last = 2.5
-        x_results = Dvector[]
-        y_results = Dvector[]
-        t.make_steps(
-            'dest_xs' => x_results, 'dest_ys' => y_results, 
+        x_results = Dvector[0]
+        y_results = Dvector[0]
+        stps = t.make_steps(
             'xs' => xs, 'ys' => ys,
             'x_first' => x_first, 'y_first' => y_first, 
             'x_last' => x_last, 'y_last' => y_last)
@@ -614,7 +726,7 @@ EOD
             t.fill_color = FloralWhite
             t.fill_frame
             t.stroke_color = Blue
-            t.append_points_to_path(x_results, y_results)
+            t.append_points_to_path(stps[0], stps[1])
             t.stroke
             t.show_marker('Xs' => xs, 'Ys' => ys, 'marker' => Bullet, 
                 'scale' => 0.6, 'color' => Red);
@@ -730,59 +842,7 @@ EOD
         t.close_path
         t.clip
     end
-    
-    def test_samples_with_contours
-        t.rescale(0.8)
-        title = 'Log Opacity'
-        levels = Array.new
-        (0..5).each { |i| levels << i + 0.4 }
-        t.subplot('right_margin' => @image_right_margin) { sampled_image(title) }
-        t.subplot('left_margin' => 0.95, 
-            'top_margin' => 0.05, 
-            'bottom_margin' => 0.05) { color_bar(title, levels) }
-        t.subplot('right_margin' => @image_right_margin) do
-            t.xaxis_type = t.yaxis_type = AXIS_WITH_TICKS_ONLY
-            t.no_title; t.no_xlabel; t.no_ylabel
-            bounds = [@eos_xmin, @eos_xmax, @eos_ymax, @eos_ymin]
-            t.show_plot(bounds) do
-                clip_press_image
-                t.stroke_color = SlateGray
-                t.line_width = 1
-                dest_xs = Dvector.new; dest_ys = Dvector.new; gaps = Array.new
-                dict = { 'dest_xs' => dest_xs, 
-                        'dest_ys' => dest_ys, 
-                        'gaps' => gaps,
-                        'xs' => @eos_logRHOs, 
-                        'ys' => @eos_logTs,
-                        'data' => @opacity_data }
-                levels.each do |level|
-                    dict['level'] = level
-                    num_xs = @eos_logRHOs.length
-                    num_ys = @eos_logTs.length
-                    legit = Dtable.new(num_xs, num_ys)
-                    num_xs.times do |ix|
-                      num_ys.times do |iy|
-                        if @opacity_data[ix,iy] < -999
-                          legit[ix,iy] = 0.0
-                        else
-                          legit[ix,iy] = 1.0
-                        end
-                      end
-                    end
-                    #dict['legit'] = legit
-                    t.make_contour(dict)
-                    if false
-                      t.append_points_to_path(dest_xs, dest_ys)
-                      t.fill_color = Green
-                      t.fill
-                    else
-                      t.append_points_with_gaps_to_path(dest_xs, dest_ys, gaps, true)
-                      t.stroke
-                    end
-                end
-            end
-        end
-    end
+
 
     def samples_with_contours
         t.rescale(0.8)
@@ -801,23 +861,64 @@ EOD
                 clip_press_image
                 t.stroke_color = SlateGray
                 t.line_width = 1
-                dest_xs = Dvector.new; dest_ys = Dvector.new; gaps = Array.new
-                dict = { 'dest_xs' => dest_xs, 
-                        'dest_ys' => dest_ys, 
-                        'gaps' => gaps,
+                gaps = Array.new
+                dict = { 'gaps' => gaps,
                         'xs' => @eos_logRHOs, 
                         'ys' => @eos_logTs,
                         'data' => @opacity_data }
                 levels.each do |level|
                     dict['level'] = level
-                    t.make_contour(dict)
-                    t.append_points_with_gaps_to_path(dest_xs, dest_ys, gaps, true)
+                    pts_array = t.make_contour(dict)
+                    t.append_points_with_gaps_to_path(pts_array[0], pts_array[1], gaps, true)
                     t.stroke
                 end
             end
         end
     end
 
+    # This plot is to demonstrate the new power of #show_axis.
+    def axes_fun
+      t.do_box_labels("Funny axes", "", "")
+      t.show_plot([-1, 19, 8, 2]) do
+        spec = {
+          'from' => [3,3],
+          'to' => [3,7],
+          'ticks_outside' => true,
+          'ticks_inside' => false,
+        }
+        t.show_axis(spec)
+        p t.axis_information(RIGHT)
+        spec2 = {
+          'from' => [12,3],
+          'to' => [12,7],
+          'ticks_outside' => true,
+          'ticks_inside' => false,
+          'major_ticks' => [4,6],
+          'labels' => ["$a$", "$b$"]
+        }
+        t.show_axis(spec2)
+        spec3 = {
+          'from' => [17,3],
+          'to' => [17,7],
+          'ticks_outside' => true,
+          'ticks_inside' => false,
+          'shift' => 1.3,
+          'labels' => ["$a$", "$b$", 'c', 'd', 'e']
+        }
+        t.show_axis(spec3)
+      end
+      t.context do 
+        t.set_bounds([-1, 19, 8, 2])
+        spec = {
+          'from' => [4,1.2],
+          'to' => [12,1.2],
+          'major_ticks' => [ 4.5, 8.8]
+        }
+        t.show_axis(spec)
+      end
+    end
+
+
 
 end
 
