@@ -35,7 +35,11 @@ class MyPlots
 #         @big_blues_scale = 11.0
         
         t.def_figure("Arrows") { arrows }
-#         t.def_figure("Reds") { reds }
+        t.def_figure("Markers") { marker_position }
+        t.def_figure("Lines") { lines_position }
+        t.def_figure("Cap") { lines_cap }
+        t.def_figure("show_marker_bbox") {show_marker_bbox}
+        t.def_figure("marker_centered") {marker_centered}
 #         t.def_figure("Greens") { greens }
 #         t.def_figure("Log_Reds") { log_reds }
 #         t.def_figure("Side_by_Side") { side_by_side }
@@ -493,6 +497,157 @@ class MyPlots
                     " are aligned together}")
       end
     end
+
+
+
+    def marker_position
+      x = Dvector[0,1,2,3,2,0,1,2,3,4,1,1]
+      y = Dvector[2,0,3,5,4,0,1,2,3,4,4,-0.5]
+      #t.do_box_labels("Arrows tests", "Position", "Blues")
+      t.show_plot([-1,4,6,-1]) do
+        t.show_marker('Xs' => x, 'Ys' => y,
+                      'marker' => Circle,
+                      'color' => Green)
+
+        t.line_width = 0.3
+        t.stroke_color = Red
+        t.append_points_to_path(x, y)
+        t.stroke
+
+        t.show_text('at' => [1,5], 
+                    'text' => '\parbox{5cm}{Please check that the ' +
+                    "symbols are centered on the lines}")
+      end
+    end
+    
+    
+    def marker_centered
+      
+      t.show_plot([-1,1,1,-1]) do
+        angle = 90
+        scale = 20
+        mrkstr = "\251" # Diamond
+        marker_info = t.marker_string_info(ZapfDingbats, mrkstr, scale)
+        t.show_marker('x' => 0.0, 'y' => 0.0, 
+                      'font' => ZapfDingbats,'string' => mrkstr, "scale" => scale, 'angle' => angle,
+                      'color' => Green)
+        t.stroke_width = 0.1
+        t.stroke_line(0,-1,0,1)
+        t.stroke_line(-1,0,1,0)
+      end
+
+    end
+    
+    
+    def show_marker_bbox
+      
+      t.show_plot([-0.2,1.7,1.3,-0.2]) do
+        angle = 0
+        scale = 20
+        mrkstr = "\251" # Diamond
+        mrkstr = "\250" # Club
+        mrkstr = "\252" # Heart
+        mrkstr = "\072" # Cross
+        mrkstr = "\053"
+        marker_info = t.marker_string_info(ZapfDingbats, mrkstr, scale)
+        t.show_marker('x' => 0.0, 'y' => 0.0, 
+                      'justification' => LEFT, 'alignment' => ALIGNED_AT_BASELINE,
+                      'font' => ZapfDingbats,'string' => mrkstr, "scale" => scale, 'angle' => angle,
+                      'color' => Green)
+        t.stroke_color = Crimson
+        dx_left = marker_info[1]
+        dy_down = marker_info[2]
+        dx_right = marker_info[3]# - 0.115
+        dy_up = marker_info[4]
+        t.stroke_line(dx_left,dy_down,dx_left,dy_up)
+        t.stroke_line(dx_right,dy_down,dx_right,dy_up)
+        t.stroke_line(dx_left,dy_down,dx_right,dy_down)
+        t.stroke_line(dx_left,dy_up,dx_right,dy_up)
+      end
+    
+    end
+
+    def lines_position
+      x = Dvector[0,1,2,3,4]
+      y = Dvector[0,1,2,3,4]
+      #t.do_box_labels("Arrows tests", "Position", "Blues")
+      t.show_plot([-1,5,8,-1]) do
+        marker = Diamond
+        angle = 0
+        scale = 1
+        t.show_marker('Xs' => x, 'Ys' => y,
+                      'marker' => marker, "scale" => scale, 'angle' => angle,
+                      'color' => Green)
+
+
+        t.line_width = 3
+        t.stroke_color = Red
+        t.append_points_to_path(x, y)
+        t.stroke
+
+
+        y.add!(1)
+        t.show_marker('Xs' => x, 'Ys' => y,
+                      'marker' => marker, "scale" => 0.2, 'angle' => angle,
+                      'color' => Green)
+
+        t.line_width = 0.2
+        t.stroke_color = Red
+        t.append_points_to_path(x, y)
+        t.stroke
+
+        y.add!(1)
+        y.reverse!
+        x.reverse!
+        t.show_marker('Xs' => x, 'Ys' => y,
+                      'marker' => marker, "scale" => scale, 'angle' => angle,
+                      'color' => Green)
+
+        t.line_width = 3
+        t.stroke_color = Red
+        t.append_points_to_path(x, y)
+        t.stroke
+
+        t.line_width = 0.1
+        t.stroke_color = Black
+        for val in x
+          t.append_points_to_path(Dvector[val,val], Dvector[0,6])
+          t.stroke
+        end
+
+        t.show_text('at' => [1,7], 
+                    'text' => '\parbox{5cm}{Please check that the ' +
+                    "symbols are centered on the lines}")
+      end
+    end
+
+    def lines_cap
+      x = Dvector[0,0,1,1]
+      y = Dvector[0,1,1,0]
+      t.show_plot([-1,4,4,-1]) do
+
+        t.line_width = 3
+        t.line_cap = LINE_CAP_BUTT
+        t.append_points_to_path(x, y)
+
+        x.add!(2)
+        t.move_to_point(2,0)
+        x.shift
+        y.shift
+
+        t.stroke_color = Red
+        t.append_points_to_path(x, y)
+        t.stroke
+
+        t.line_width = 0.1
+        t.stroke_color = Green
+        t.show_polyline([-1,4], [0,0])
+
+        t.show_text('at' => [1,3], 
+                    'text' => '\parbox{5cm}{Please check that the ' +
+                    "red lines don't cross the green one}")
+      end
+    end
     
     def sampled_splines
         t.do_box_labels("Sampled Splines", "Position", "Average Count")
