@@ -25,6 +25,9 @@
 
 #include "figures.h"
 #include "pdfs.h"
+#include "wrappers.h"
+#include "dvector.h"
+#include "dtable.h"
 #include "flate.h"
 
 #include <symbols.h>
@@ -32,46 +35,47 @@
 
 #include <stdio.h>
 
+
+#define DBL_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : rb_float_new(p->attr); } \
+static VALUE FM_##attr##_set(VALUE fmkr, VALUE val) { \
+   int ierr=0; FM *p = Get_FM(fmkr,&ierr); if (ierr != 0) RETURN_NIL; VALUE v = rb_Float(val); p->attr = NUM2DBL(v); return val; }
+
+#define INT_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : INT2FIX(p->attr); } \
+static VALUE FM_##attr##_set(VALUE fmkr, VALUE val) { \
+   int ierr=0; FM *p = Get_FM(fmkr,&ierr); if (ierr != 0) RETURN_NIL; VALUE v = rb_Integer(val); p->attr = NUM2INT(v); return val; }
+
+#define VAL_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : p->attr; } \
+static VALUE FM_##attr##_set(VALUE fmkr, VALUE val) { \
+   int ierr=0; FM *p = Get_FM(fmkr,&ierr); if (ierr != 0) RETURN_NIL; p->attr = val; return val; }
+
+#define BOOL_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : ((p->attr)? Qtrue : Qfalse); } \
+static VALUE FM_##attr##_set(VALUE fmkr, VALUE val) { \
+   int ierr=0; FM *p = Get_FM(fmkr,&ierr); if (ierr != 0) RETURN_NIL; p->attr = (val != Qfalse); return val; }
+
+#define RO_DBL_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : rb_float_new(p->attr); }
+
+#define RO_INT_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : INT2FIX(p->attr); }
+
+#define RO_VAL_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : p->attr; }
+
+#define RO_BOOL_ATTR(attr) \
+static VALUE FM_##attr##_get(VALUE fmkr) { int ierr=0; FM *p = Get_FM(fmkr,&ierr); return (ierr != 0)? OBJ_NIL : ((p->attr)? Qtrue : Qfalse); }
+
+
 char *data_dir = NULL;
 
-VALUE cFM; /* the Tioga/FigureMaker class object */
+OBJ_PTR cFM; /* the Tioga/FigureMaker class object */
 
-static void FM_mark(FM *p) { /* all of the VALUEs in the FM struct should be marked */
-   rb_gc_mark(p->stroke_color);
-   rb_gc_mark(p->fill_color);
-   rb_gc_mark(p->line_type);
-   rb_gc_mark(p->title);
-   rb_gc_mark(p->title_color);
-   rb_gc_mark(p->xlabel);
-   rb_gc_mark(p->xlabel_color);
-   rb_gc_mark(p->ylabel);
-   rb_gc_mark(p->ylabel_color);
-   rb_gc_mark(p->xaxis_stroke_color);
-   rb_gc_mark(p->xaxis_locations_for_major_ticks);
-   rb_gc_mark(p->xaxis_locations_for_minor_ticks);
-   rb_gc_mark(p->yaxis_stroke_color);
-   rb_gc_mark(p->yaxis_locations_for_major_ticks);
-   rb_gc_mark(p->yaxis_locations_for_minor_ticks);
-   rb_gc_mark(p->fm);
-}
-
-static void FM_free(FM *p) {
-   free(p);
-}
-
-static VALUE FM_alloc(VALUE klass) {
-   FM *p;
-   VALUE ary = Data_Make_Struct(klass, FM, FM_mark, FM_free, p);
-   Initialize_Figure(ary);
-   p->fm = ary;
-   return ary;
-}
-
-bool Is_FM(VALUE fmkr) { return ( TYPE(fmkr) == T_DATA && RDATA(fmkr)->dfree == (RUBY_DATA_FUNC)FM_free ); }
-
-FM *Get_FM(VALUE fmkr) {
-   FM *p;
-   Data_Get_Struct(fmkr, FM, p);
+FM *Get_FM(OBJ_PTR fmkr, int *ierr) {
+   FM *p = (FM *)Dvector_Data_for_Write(Get_fm_data_attr(fmkr, ierr), NULL);
+   if (*ierr != 0) RAISE_ERROR("FigMkr is missing @fm_data", ierr);
    return p;
 }
 
@@ -129,19 +133,15 @@ FM *Get_FM(VALUE fmkr) {
    
 /* graphics attribute accessors */
    RO_DBL_ATTR(default_line_scale)        
-   RO_VAL_ATTR(stroke_color)
-   RO_VAL_ATTR(fill_color)
    RO_DBL_ATTR(line_width)        
    RO_INT_ATTR(line_cap)
    RO_INT_ATTR(line_join)
    RO_DBL_ATTR(miter_limit)
    RO_DBL_ATTR(stroke_opacity)
    RO_DBL_ATTR(fill_opacity)
-   RO_VAL_ATTR(line_type)
    
 /* Title */
    RO_BOOL_ATTR(title_visible)
-   VAL_ATTR(title)
    INT_ATTR(title_side)
    DBL_ATTR(title_position)
    DBL_ATTR(title_scale)
@@ -149,11 +149,10 @@ FM *Get_FM(VALUE fmkr) {
    DBL_ATTR(title_angle)
    INT_ATTR(title_alignment)
    INT_ATTR(title_justification)
-   VAL_ATTR(title_color)
+   //VAL_ATTR(title_color)
     
 /* X label */
    RO_BOOL_ATTR(xlabel_visible)
-   VAL_ATTR(xlabel)
    DBL_ATTR(xlabel_position)
    DBL_ATTR(xlabel_scale)
    DBL_ATTR(xlabel_shift)
@@ -161,11 +160,9 @@ FM *Get_FM(VALUE fmkr) {
    INT_ATTR(xlabel_side)
    INT_ATTR(xlabel_alignment)
    INT_ATTR(xlabel_justification)
-   VAL_ATTR(xlabel_color)
     
 /* Y label */
    RO_BOOL_ATTR(ylabel_visible)
-   VAL_ATTR(ylabel)
    DBL_ATTR(ylabel_position)
    DBL_ATTR(ylabel_scale)
    DBL_ATTR(ylabel_shift)
@@ -173,14 +170,12 @@ FM *Get_FM(VALUE fmkr) {
    INT_ATTR(ylabel_side)
    INT_ATTR(ylabel_alignment)
    INT_ATTR(ylabel_justification)
-   VAL_ATTR(ylabel_color)
     
 /* X axis */
    RO_BOOL_ATTR(xaxis_visible)
    INT_ATTR(xaxis_loc)
    INT_ATTR(xaxis_type)
    DBL_ATTR(xaxis_line_width)
-   VAL_ATTR(xaxis_stroke_color)
    DBL_ATTR(xaxis_major_tick_width)
    DBL_ATTR(xaxis_minor_tick_width)
    DBL_ATTR(xaxis_major_tick_length)
@@ -191,11 +186,8 @@ FM *Get_FM(VALUE fmkr) {
    DBL_ATTR(xaxis_tick_interval)
    DBL_ATTR(xaxis_min_between_major_ticks)
    INT_ATTR(xaxis_number_of_minor_intervals)
-   VAL_ATTR(xaxis_locations_for_major_ticks)
-   VAL_ATTR(xaxis_locations_for_minor_ticks)
    BOOL_ATTR(xaxis_use_fixed_pt)
    INT_ATTR(xaxis_digits_max)
-   VAL_ATTR(xaxis_tick_labels)
    INT_ATTR(xaxis_numeric_label_decimal_digits)
    DBL_ATTR(xaxis_numeric_label_scale)
    DBL_ATTR(xaxis_numeric_label_shift)
@@ -214,7 +206,6 @@ FM *Get_FM(VALUE fmkr) {
    INT_ATTR(yaxis_loc)
    INT_ATTR(yaxis_type)
    DBL_ATTR(yaxis_line_width)
-   VAL_ATTR(yaxis_stroke_color)
    DBL_ATTR(yaxis_major_tick_width)
    DBL_ATTR(yaxis_minor_tick_width)
    DBL_ATTR(yaxis_major_tick_length)
@@ -225,11 +216,8 @@ FM *Get_FM(VALUE fmkr) {
    DBL_ATTR(yaxis_tick_interval)
    DBL_ATTR(yaxis_min_between_major_ticks)
    INT_ATTR(yaxis_number_of_minor_intervals)
-   VAL_ATTR(yaxis_locations_for_major_ticks)
-   VAL_ATTR(yaxis_locations_for_minor_ticks)
    BOOL_ATTR(yaxis_use_fixed_pt)
    INT_ATTR(yaxis_digits_max)
-   VAL_ATTR(yaxis_tick_labels)
    INT_ATTR(yaxis_numeric_label_decimal_digits)
    DBL_ATTR(yaxis_numeric_label_scale)
    DBL_ATTR(yaxis_numeric_label_shift)
@@ -262,12 +250,26 @@ FM *Get_FM(VALUE fmkr) {
 /* Warning on non-ok numbers */
    BOOL_ATTR(croak_on_nonok_numbers)
 
+bool Get_initialized() {
+   OBJ_PTR v = rb_cv_get(cFM, "@@initialized");
+   return v != OBJ_FALSE && v != OBJ_NIL;
+}
+
+void Set_initialized() {
+   rb_cv_set(cFM, "@@initialized", OBJ_TRUE);
+}
+
+static void Set_fm_data_size() {
+   rb_cv_set(cFM, "@@fm_data_size", Integer_New(1 + (sizeof(FM) / sizeof(double)))); 
+   // size is number of doubles needed to hold FM data
+}
+
 #define attr_reader(attr) rb_define_method(cFM, #attr , FM_##attr##_get, 0);
 #define attr_writer(attr) rb_define_method(cFM, #attr "=", FM_##attr##_set, 1);
 #define attr_accessors(attr) attr_reader(attr) attr_writer(attr)
 
 void Init_FigureMaker(void) { 
-     /* called by Ruby when the extension is loaded */
+   /* called by Ruby when the extension is loaded */
 
    /* this function has been modified by Vincent Fourmond for the splitting
       out of libraries more general than Tioga */
@@ -286,12 +288,12 @@ void Init_FigureMaker(void) {
    rb_require("Flate");
 
    
-   VALUE mTioga = rb_define_module("Tioga");
+   OBJ_PTR mTioga = rb_define_module("Tioga");
 
 
    /* and now, we need to import Dobjects and Flate modules*/
-   VALUE mDobjects = rb_define_module("Dobjects");
-   VALUE mFlate = rb_define_module("Flate");
+   OBJ_PTR mDobjects = rb_define_module("Dobjects");
+   OBJ_PTR mFlate = rb_define_module("Flate");
    rb_include_module(mTioga, mDobjects);
    rb_include_module(mTioga, mFlate);
 
@@ -304,13 +306,13 @@ void Init_FigureMaker(void) {
    rb_include_module(cFM, mDobjects);
    rb_include_module(cFM, mFlate);
 
-   rb_define_alloc_func(cFM, FM_alloc);
    Init_IDs();
    Init_Font_Dictionary();
    rb_define_method(cFM, "private_make", FM_private_make, 2);
    rb_define_method(cFM, "get_save_filename", FM_get_save_filename, 1);
    rb_define_method(cFM, "private_make_portfolio", FM_private_make_portfolio, 3);
-   
+   rb_define_method(cFM, "private_init_fm_data", FM_private_init_fm_data, 0);
+
 /* page attribute accessors */
    attr_reader(root_figure)
    attr_reader(in_subplot)
@@ -364,15 +366,16 @@ void Init_FigureMaker(void) {
    attr_accessors(line_width)
    attr_accessors(line_cap)
    attr_accessors(line_join)
+   rb_define_method(cFM, "line_type_set", FM_line_type_set, 1);
    attr_accessors(miter_limit)
    attr_accessors(stroke_opacity)
    attr_accessors(fill_opacity)
-   attr_accessors(line_type)
 /* croak on non ok */
    attr_accessors(croak_on_nonok_numbers)
 
 /* methods */
-   rb_define_method(cFM, "private_context", FM_private_context, 1);
+   rb_define_method(cFM, "pdf_gsave", FM_pdf_gsave, 0);
+   rb_define_method(cFM, "pdf_grestore", FM_pdf_grestore, 0);
    rb_define_method(cFM, "private_set_bounds", FM_private_set_bounds, 4);
    rb_define_method(cFM, "private_set_subframe", FM_private_set_subframe, 4);
    rb_define_method(cFM, "doing_subfigure", FM_doing_subfigure, 0);
@@ -415,17 +418,20 @@ void Init_FigureMaker(void) {
    rb_define_method(cFM, "convert_output_to_figure_x", FM_convert_output_to_figure_x, 1);
    rb_define_method(cFM, "convert_output_to_figure_y", FM_convert_output_to_figure_y, 1);
    rb_define_method(cFM, "convert_output_to_figure_dx", FM_convert_output_to_figure_dx, 1);
-   rb_define_method(cFM, "convert_output_to_figuret_dy", FM_convert_output_to_figure_dy, 1);
+   rb_define_method(cFM, "convert_output_to_figure_dy", FM_convert_output_to_figure_dy, 1);
    rb_define_method(cFM, "convert_to_degrees", FM_convert_to_degrees, 2);
 /* text */
    rb_define_method(cFM, "private_set_default_font_size", FM_private_set_default_font_size, 1);
    rb_define_method(cFM, "rescale_text", FM_rescale_text, 1);
-   rb_define_method(cFM, "show_rotated_text", FM_show_rotated_text, 8);
-   rb_define_method(cFM, "show_rotated_label", FM_show_rotated_label, 7);
+   rb_define_method(cFM, "show_rotated_text", FM_show_rotated_text, 9);
+   rb_define_method(cFM, "show_rotated_label", FM_show_rotated_label, 8);
    rb_define_method(cFM, "check_label_clip", FM_check_label_clip, 2);
+/* text measurements */
+   rb_define_method(cFM, "private_save_measure", FM_private_save_measure, 4);
 /* path construction */
    rb_define_method(cFM, "move_to_point", FM_move_to_point, 2);
    rb_define_method(cFM, "append_point_to_path", FM_append_point_to_path, 2);
+   rb_define_method(cFM, "bezier_control_points", FM_bezier_control_points, 6);
    rb_define_method(cFM, "append_curve_to_path", FM_append_curve_to_path, 6);
    rb_define_method(cFM, "close_path", FM_close_path, 0);
    rb_define_method(cFM, "append_points_to_path", FM_append_points_to_path, 2);
@@ -502,6 +508,7 @@ void Init_FigureMaker(void) {
    rb_define_method(cFM, "doing_subplot", FM_doing_subplot, 0);
    rb_define_method(cFM, "show_axis", FM_show_axis, 1);
    rb_define_method(cFM, "show_edge", FM_show_edge, 1);
+   rb_define_method(cFM, "axis_information", FM_axis_information, 1);
    rb_define_method(cFM, "no_title", FM_no_title, 0);
    rb_define_method(cFM, "no_xlabel", FM_no_xlabel, 0);
    rb_define_method(cFM, "no_ylabel", FM_no_ylabel, 0);
@@ -512,13 +519,12 @@ void Init_FigureMaker(void) {
    rb_define_method(cFM, "no_top_edge", FM_no_top_edge, 0);
    rb_define_method(cFM, "no_bottom_edge", FM_no_bottom_edge, 0);
 /* makers */
-   rb_define_method(cFM, "private_make_contour", FM_private_make_contour, 9);
-   rb_define_method(cFM, "private_make_spline_interpolated_points", FM_private_make_spline_interpolated_points, 6);
-   rb_define_method(cFM, "private_make_steps", FM_private_make_steps, 8);
+   rb_define_method(cFM, "private_make_contour", FM_private_make_contour, 7);
+   rb_define_method(cFM, "private_make_spline_interpolated_points", FM_private_make_spline_interpolated_points, 5);
+   rb_define_method(cFM, "private_make_steps", FM_private_make_steps, 6);
    
 /* Title */
    attr_reader(title_visible)
-   attr_accessors(title)
    attr_accessors(title_side)
    attr_accessors(title_position)
    attr_accessors(title_scale)
@@ -530,7 +536,6 @@ void Init_FigureMaker(void) {
     
 /* X label */
    attr_reader(xlabel_visible)
-   attr_accessors(xlabel)
    attr_accessors(xlabel_position)
    attr_accessors(xlabel_scale)
    attr_accessors(xlabel_shift)
@@ -542,7 +547,6 @@ void Init_FigureMaker(void) {
     
 /* Y label */
    attr_reader(ylabel_visible)
-   attr_accessors(ylabel)
    attr_accessors(ylabel_position)
    attr_accessors(ylabel_scale)
    attr_accessors(ylabel_shift)
@@ -568,11 +572,8 @@ void Init_FigureMaker(void) {
    attr_accessors(xaxis_tick_interval)
    attr_accessors(xaxis_min_between_major_ticks)
    attr_accessors(xaxis_number_of_minor_intervals)
-   attr_accessors(xaxis_locations_for_major_ticks)
-   attr_accessors(xaxis_locations_for_minor_ticks)
    attr_accessors(xaxis_use_fixed_pt)
    attr_accessors(xaxis_digits_max)
-   attr_accessors(xaxis_tick_labels)
    attr_accessors(xaxis_numeric_label_decimal_digits)
    attr_accessors(xaxis_numeric_label_scale)
    attr_accessors(xaxis_numeric_label_shift)
@@ -602,11 +603,8 @@ void Init_FigureMaker(void) {
    attr_accessors(yaxis_tick_interval)
    attr_accessors(yaxis_min_between_major_ticks)
    attr_accessors(yaxis_number_of_minor_intervals)
-   attr_accessors(yaxis_locations_for_major_ticks)
-   attr_accessors(yaxis_locations_for_minor_ticks)
    attr_accessors(yaxis_use_fixed_pt)
    attr_accessors(yaxis_digits_max)
-   attr_accessors(yaxis_tick_labels)
    attr_accessors(yaxis_numeric_label_decimal_digits)
    attr_accessors(yaxis_numeric_label_scale)
    attr_accessors(yaxis_numeric_label_shift)
@@ -636,11 +634,13 @@ void Init_FigureMaker(void) {
 /* Debugging */
    attr_accessors(debug_verbosity_level)
       
+      
+   Set_fm_data_size(); // must set this before create a FigureMaker instance
    rb_require("Tioga/FigMkr.rb");
    /* We now need to import the symbols */
 
    /* imports from Dvector */
-   VALUE cDvector = rb_define_class_under(mDobjects, "Dvector", rb_cObject);
+   OBJ_PTR cDvector = rb_define_class_under(mDobjects, "Dvector", rb_cObject);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Create);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Data_Resize);
    RB_IMPORT_SYMBOL(cDvector, Dvector_Data_Replace);
@@ -656,7 +656,7 @@ void Init_FigureMaker(void) {
    RB_IMPORT_SYMBOL(mFlate, flate_expand);
 
    /* imports from Dtable */
-   VALUE cDtable = rb_define_class_under(mDobjects, "Dtable", rb_cObject);
+   OBJ_PTR cDtable = rb_define_class_under(mDobjects, "Dtable", rb_cObject);
    RB_IMPORT_SYMBOL(cDtable, Dtable_Ptr);
    RB_IMPORT_SYMBOL(cDtable, Read_Dtable);
 }
